import time

# Import the preset TestFixture.
from .Frames_capture_fixture import FixtureFramesCapture, ChannelFactory

from xtr import logging, Severity
logger = logging.getLogger(__name__)


# Create the test case class, that inherits from the preset TestFixture.
class TestCaptureCanFrames(FixtureFramesCapture):

    # This method will be executed just before
    # the test case method execution.
    def setUp(self):
        logger.info("Setup test case")

    # This method will be executed just after
    # the test case method execution.
    def tearDown(self):
        logger.info("Tear down test case")

    # Create the test case method that holds the main test steps.
    def test_capture_can_frames(self):

        # Define frame properties.
        # The values provided below are arbitrary examples.
        # They do not belong to any actual in vehicle communication.
        channel_name = 'B1_CAN'
        frame_id = 21  # 0x15 in hexadecimal
        payload = [0x1, 0x2, 0x3, 0x48]

        can_ch = ChannelFactory().get_can_channel(channel_name, False)

        can_fr = can_ch.get_frame(frame_id)

        # Start listening on the can channel.
        can_ch.listener.start_listening()

        # Log any useful message.
        logger.info("Transmit a can frame")

        # Transmit the frame.
        can_fr.transmitter.transmit_frame(payload)

        # Pause the program for 3 seconds.
        time.sleep(3)

        # Stop listening on the can channel
        can_ch.listener.stop_listening()

        q = can_ch.listener.get_queue()

        # Confirm that the queue is not Null object,
        # meaning there is no error in creating the queue.
        self.assertTrue(q is not None, Severity.BLOCKER,
                        "Error when listening to the can bus")

        # Confirm that queue is not empty meaning messages are received.
        self.assertTrue(not q.empty(), Severity.BLOCKER,
                        "Can frame is not received")

        event = q.queue[0]
        logger.info(repr(event))

        logger.info(f"{event.channel_name}")
        logger.info(f"{event.frame_id}")
